# Crosby

Allows for quality checks on `ActionMailer` emails when migrating to new systems.

http://en.wikipedia.org/wiki/Philip_B._Crosby

```
Crosby's response to the quality crisis was the principle of
"doing it right the first time" (DIRFT). He also included four major principles:

The definition of quality is conformance to requirements
  (requirements meaning both the product and the customer's requirements)
The system of quality is prevention
The performance standard is zero defects (relative to requirements)
The measurement of quality is the price of nonconformance
```

This is a work in progress which came out of a need to compare emails generated
on our 'nextjenn' platform to emails generated in our legacy codebase.

## Installation

Add this line to your application's Gemfile:

```ruby
gem "crosby"
```

And then execute:

    $ bundle

Or install it yourself as:

    $ gem install crosby

## Configuration

Crosby has 2 configurable values, `app_name` and `export_path` with the
following defaults:

```ruby
Crosby.config
# => { :app_name => "6f292f17", :export_path => "tmp/crosby" }
```

NOTE: The value of `app_name` is randomized and will not stay consistent across
application loads unless specifically set. Config options can be set via a
block.

```ruby
Crosby.config do |c|
  c.app_name = "ExampleApp"
  c.export_path = "../crosby_files"
end
```

## Usage

### Basic Usage

Crosby exports a slightly curated version of `Mail::Message` objects produced
by `ActionMailer` subclasses to a `.crosby` file. The class responsible for
producing this file is `Crosby::Exporter`.

```ruby
exporter = Crosby::Exporter.new(
  MailerClass, :method, 'a-unique-id'[, arg1[, arg2[, ...]]]
)

exporter.export
```

Also available is the `Crosby::Exporter#to_s` method which will return the
`.crosby` file's contents as a string.

```ruby
exporter.to_s
```

### RSpec

`Crosby::RSpecHelper` works with RSpec 1, 2 & 3, and adds the `#crosby_email`
helper method. You can use this method to create `.crosby` files inside your
existing tests.

```ruby
# spec_helper.rb

require "crosby/rspec_helper"

Crosby.config do |c|
  # ...
end
```

```ruby
# spec/mailers/example_mailer_spec.rb
require "spec_helper"

describe ExampleMailer do
  context "#notify" do
    it "should send a notification email" do
      mailer = ExampleMailer.notify(arg1, arg1)

      # becomes
      mailer = crosby_email(ExampleMailer, :notify, arg1, arg2)

      # ...
    end
  end

  # ...
end
```

In the above example, Crosby will generate a uuid based on the mailer class,
method name and argument values. While this is fine, it may be helpful to
implicitly set the uuid. You can do this by setting the `@crosby_uuid` instance
variable in your test.

```ruby
# spec/mailers/example_mailer_spec.rb
require "spec_helper"

describe ExampleMailer do
  context "#notify" do
    it "should send a notification email" do
      @crosby_uuid = "example_mailer_notify"
      mailer = crosby_email(ExampleMailer, :notify, arg1, arg2)

      # ...
    end
  end

  # ...
end
```

#### What about CI? Can I skip Crosby?

If you'd like to skip the creation of `.crosby` output files, simply set
`ENV["SKIP_CROSBY"]=true` and neither the export directory nor the output file
will be created.

### Generating a diff

`Crosby::Diff` is responsible for comparing files of the same uuid to eachother.
There must be at least 2 files with the same uuid present in the configured
directory for a diff to be generated. If there are more than 2 files for a
given uuid, a diff will be generated for each unique pair of files.

```ruby
Crosby::Diff.compare("example_mailer_notify")
# => [diff_str, ...]
```

`Crosby::Diff.compare` returns an array of diff strings, one for each file pair.
When no pairs are present, `compare` returns an empty array.

```ruby
Crosby::Diff.compare("not-an-existing-uuid")
# => []
```

It also accepts an optional block, to be called once per diff string.

```ruby
Crosby::Diff.compare("example_mailer_notify") { |diff| puts diff }
# diff output
# => [diff_str, ...]
```

`Crosby::Diff.compare` and `Crosby::Diff.new` both accept an options hash to
override `Crosby.config` settings.

```ruby
Crosby::Diff.compare("example_mailer_notify", :export_path => "../crosby_files")
Crosby::Diff.new("example_mailer_notify", :export_path => "../crosby_files")
```

`Crosby::Diff` uses the excellent [Diffy gem](https://github.com/samg/diffy)
behind the scenes. If you would like access to the `Diffy::Diff` objects, you
can access them through the `Crosby::Diff#diffs` method.

```ruby
cdiff = Crosby::Diff.new("example_mailer_notify")
cdiff.diffs
# => [ [#<Diffy::Diff:0x00000000>, #<Diffy::Diff:0x00000001], [..., ...], ... ]
```

### Rake

Crosby provides a simple `rake crosby:diff` command which outputs diffs for all
of the uuids represented by files in the directory specified at
`Crosby.config[:export_path]`.

```ruby
# Rakefile
require "crosby/tasks"

Crosby.config do |c|
  c.export_path = "../crosby_files"
end
```

```sh
bundle exec rake crosby:diff
```

## Contributing

1. Fork it ( https://github.com/[my-github-username]/crosby/fork )
2. Create your feature branch (`git checkout -b my-new-feature`)
3. Commit your changes (`git commit -am 'Add some feature'`)
4. Push to the branch (`git push origin my-new-feature`)
5. Create a new Pull Request
